package org.ow2.orchestra.designer.commands {
  import com.adobe.cairngorm.business.ServiceLocator;
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;
  import com.adobe.cairngorm.control.CairngormEventDispatcher;

  import mx.collections.ArrayCollection;
  import mx.collections.ListCollectionView;
  import mx.managers.CursorManager;
  import mx.rpc.AsyncToken;
  import mx.rpc.Responder;
  import mx.rpc.events.FaultEvent;
  import mx.rpc.events.ResultEvent;
  import mx.rpc.remoting.RemoteObject;

  import org.ow2.orchestra.designer.control.DesignerMessageCodes;
  import org.ow2.orchestra.designer.events.FileNavigatorEvent;
  import org.ow2.orchestra.designer.events.ProcessEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.ProcessEditorModel;
  import org.ow2.orchestra.designer.utils.ErrorManager;
  import org.ow2.orchestra.designer.vo.FileNavigatorItem;
  import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
  import org.ow2.orchestra.designer.vo.FileNavigatorProject;

  /**
   * Delete the selected process
   */
  public class FileNavigatorItemDeleteCMD implements ICommand {
    private var item:FileNavigatorItem;

    public function FileNavigatorItemDeleteCMD() {
    }

    /**
     *
     * @param event: FileNavigatorEvent
     *
     */
    public function execute(event:CairngormEvent):void {
      item = (event as FileNavigatorEvent).item;
      // Dispatch an event to close the process
      if (item is FileNavigatorProcess) {
        var process:FileNavigatorProcess = item as FileNavigatorProcess;
        var processEditorModel:ProcessEditorModel = DesignerModelLocator.getInstance().getOpenProcessByFilePath(process);
        if (processEditorModel != null) {
          var evt:ProcessEvent = new ProcessEvent(ProcessEvent.PROCESS_CLOSE, processEditorModel);
          CairngormEventDispatcher.getInstance().dispatchEvent(evt);
        }
      } else if (item is FileNavigatorProject) {
        //Dispatch an event to delete all included processes.
        var project:FileNavigatorProject = item as FileNavigatorProject;
        var projectProcessesList:ListCollectionView = new ArrayCollection();
        projectProcessesList.addAll(project.processes);
        for each (var projectProcess:FileNavigatorProcess in projectProcessesList) {
          var closeProcessEvent:FileNavigatorEvent = new FileNavigatorEvent(FileNavigatorEvent.ITEM_DELETE, projectProcess);
          CairngormEventDispatcher.getInstance().dispatchEvent(closeProcessEvent);
        }
      }
      // delete process
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");
      var token:AsyncToken = remote.deleteItem(item);
      token.addResponder(new Responder(deleteItemResult, fault));

      CursorManager.setBusyCursor();
    }

    private function deleteItemResult(event:ResultEvent):void {
      if (!event.result) {
        ErrorManager.getInstance().showError("An error occurs during process deletion.", DesignerMessageCodes.PROCESS_DELETION_ERROR);
      }
      // reload tree model
      if (item is FileNavigatorProcess) {
        new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL, (item as FileNavigatorProcess).project).dispatch();
      }
      else {
        new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL, item).dispatch();
      }
    }

    private function fault(event:FaultEvent):void {
      CursorManager.removeBusyCursor();
      ErrorManager.getInstance().showError("Remote invoke error during process deletion: " + event.message);
    }
  }
}

