package org.ow2.orchestra.designer.commands {
  import com.adobe.cairngorm.business.ServiceLocator;
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.rpc.AsyncToken;
  import mx.rpc.Responder;
  import mx.rpc.events.FaultEvent;
  import mx.rpc.events.ResultEvent;
  import mx.rpc.remoting.RemoteObject;

  import org.ow2.orchestra.designer.events.FileNavigatorEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.FileNavigatorModel;
  import org.ow2.orchestra.designer.utils.ErrorManager;
  import org.ow2.orchestra.designer.vo.FileNavigatorProject;

  /**
   * Create a new project in fileNavigator.
   */
  public class NewProjectCMD implements ICommand {
    private var project:FileNavigatorProject;

    public function NewProjectCMD() {
    }

    /**
     * @param event {projectName}
     */
    public function execute(event:CairngormEvent):void {
      var fileNavigatorEvent:FileNavigatorEvent = event as FileNavigatorEvent;
      project = fileNavigatorEvent.item as FileNavigatorProject;
      var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;

      if (fileNavigatorViewModel.projectExist(project.name)) {
        ErrorManager.getInstance().showError("This project '" + project.name + "' already exists.");
      }

      else {
        var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");
        var token:AsyncToken = remote.createNewProject(project);
        token.addResponder(new Responder(result, fault));
      }
    }

    private function fault(event:FaultEvent):void {
      ErrorManager.getInstance().showError("Remote invoke error: " + event.message);
    }

    private function result(event:ResultEvent):void {
      // Refresh navigator
      new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL).dispatch();
    }
  }
}

