/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands {
  import com.adobe.cairngorm.business.ServiceLocator;
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.controls.Alert;
  import mx.rpc.AsyncToken;
  import mx.rpc.Responder;
  import mx.rpc.events.FaultEvent;
  import mx.rpc.events.ResultEvent;
  import mx.rpc.remoting.RemoteObject;

  import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
  import org.ow2.orchestra.designer.events.FileNavigatorEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.FileNavigatorModel;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
  import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
  import org.ow2.orchestra.designer.vo.FileNavigatorProject;

  import mx.managers.CursorManager;

  /**
   * To open a file
   * @author lizq
   *
   */
  public class ProcessOpenCMD implements ICommand {

    private var project:FileNavigatorProject;
    private var process:FileNavigatorProcess;

    public function ProcessOpenCMD() {
      super();
    }

    /**
     * Open a new process file or select the process if already opened.
     * @param event {filePath}
     */
    public function execute(event:CairngormEvent):void {
      trace("Command.ProcessOpenCMD");
      // save file path to store into model
      var fileNavigatorEvent:FileNavigatorEvent = event as FileNavigatorEvent;
      process = fileNavigatorEvent.item as FileNavigatorProcess;
      // check if processModel is already opened
      var processEditorModel:ProcessEditorModel =
              DesignerModelLocator.getInstance().getOpenProcessByFilePath(process);
      if (processEditorModel != null) {
        // select process editor model if exists
        DesignerModelLocator.getInstance().selectedProcessEditor = processEditorModel;
      }
      else {
        // read process model from server file
        // call open file on remote server
        var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");

        CursorManager.setBusyCursor();
        var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;
        // disable double click into FileNavigator to prevent multiple opening of the same process
        fileNavigatorViewModel.isEnabledDoubleClick = false;

        var token:AsyncToken = remote.openFile(process);
        // add fonctions to call on result
        token.addResponder(new Responder(openFileResult, fault));
      }
    }

    private function openFileResult(event:ResultEvent):void {
      var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;

      CursorManager.removeBusyCursor();
      fileNavigatorViewModel.isEnabledDoubleClick = true;

      // create process model
      var processEditorModel:ProcessEditorModel = new ProcessEditorModel();
      processEditorModel.fileNavigatorProcess = process;
      processEditorModel.processModel.name = process.name;

      processEditorModel.processModel = event.result as ProcessModel;

      // add process model to model locator
      DesignerModelLocator.getInstance().openedProcesses.addItem(processEditorModel);
      // select process model
      DesignerModelLocator.getInstance().selectedProcessEditor = processEditorModel;

    }

    private function fault(event:FaultEvent):void {
      var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;

      CursorManager.removeBusyCursor();
      // allow now opening process with double click on FileNavigator
      fileNavigatorViewModel.isEnabledDoubleClick = true;

      Alert.show("Remote invoke error: " + event.message);
    }

  }
}
