/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.utils {
  import com.adobe.cairngorm.CairngormError;
  import com.adobe.cairngorm.CairngormMessageCodes;

  import mx.controls.Alert;

  /**
   * Centralize traitment of showing applicatives errors
   *
   */

  public class ErrorManager {

    public const LEVEL_ALERT:int = 0;    //default
    public const LEVEL_TRACE:int = 1;
    public const LEVEL_EXCEPTION:int = 2;

    private var levelInfoCurrent:int = LEVEL_ALERT;

    /**
     * Singleton Pattern.
     */
    private static var instance:ErrorManager;

    public static function getInstance():ErrorManager {
      if (instance == null) {
        ErrorManager.setInstance(new ErrorManager());
      }

      return instance;
    }

    public static function setInstance(obj:ErrorManager):void {
      if (instance != null) {
        throw new CairngormError(
                CairngormMessageCodes.SINGLETON_EXCEPTION, "ErrorManager");
      }
      instance = obj;
    }

    public function ErrorManager() {
      super();
    }

    /**
     * Change the current level of showingError
     *
     */
    public function changeLevelInfo(level:int):Boolean {
      if (level == LEVEL_ALERT || level == LEVEL_TRACE || level == LEVEL_EXCEPTION) {
        this.levelInfoCurrent = level;
        return true;

      } else {
        return false;
      }
    }

    public function showError(message:String, codeError:String = null):void {
      if (levelInfoCurrent == LEVEL_ALERT) {
        Alert.show(message);
      }
      else if (levelInfoCurrent == LEVEL_TRACE) {
        trace(message);
      }
      else if (levelInfoCurrent == LEVEL_EXCEPTION) {
        throw new CairngormError(codeError, message);
      }
    }
  }
}
