package org.ow2.orchestra.designer.view.fileNavigator {
  import com.adobe.cairngorm.control.CairngormEventDispatcher;

  import flash.events.Event;
  import flash.events.KeyboardEvent;
  import flash.events.MouseEvent;
  import flash.ui.Keyboard;

  import mx.binding.utils.BindingUtils;
  import mx.controls.Tree;
  import mx.core.Application;
  import mx.events.CloseEvent;
  import mx.events.FlexEvent;
  import mx.events.ListEvent;
  import mx.managers.CursorManager;
  import mx.managers.PopUpManager;

  import org.ow2.orchestra.designer.events.FileNavigatorEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.view.DeleteConfirmWindow;
  import org.ow2.orchestra.designer.vo.FileNavigatorItem;
  import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
  import org.ow2.orchestra.designer.vo.FileNavigatorProject;

  /**
   * The file navigator view.
   */
  public class FileNavigatorView extends Tree {
    /**
     * The source for "project" image data binding.
     */
    [Bindable]
    [Embed(source="/../assets/icon/accordion/project.gif")]
    private static var projectIcon:Class;

    /**
     * The source for "Bpmn" image data binding.
     */
    [Bindable]
    [Embed(source="/../assets/icon/accordion/bpmn.gif")]
    private static var bpmnFileIcon:Class;

    private var deleteConfirmWindow:DeleteConfirmWindow;

    private var _isEnabledDoubleClick:Boolean = true;

    /**
     * Initialize the file navigator view.
     */
    public function FileNavigatorView() {
      super();

      this.x = 0;
      this.y = 0;
      this.percentHeight = 100;
      this.percentWidth = 100;
      this.iconFunction = iconFun;
      // associate with the specific treeRenderer
      //this.itemRenderer=new ClassFactory(ItemRendererTreeFileNavigator);

      this.dataDescriptor = new FileNavigatorDataDescriptor();
      this.dataProvider = DesignerModelLocator.getInstance().fileNavigatorModel.projects;
      this.doubleClickEnabled = true;

      this.showRoot = true;
      this.labelField = "name";
      this.dragEnabled = true;
      this.dropEnabled = false;

      CursorManager.setBusyCursor();
      this.addEventListener(FlexEvent.CREATION_COMPLETE, initFiles);

      this.initEventListener();
    }

    private function iconFun(item:Object):Class {
      if (item is FileNavigatorProject) {
        return projectIcon;
      }
      else if (item is FileNavigatorProcess) {
        return bpmnFileIcon;
      }
      return null;
    }

    private function initEventListener():void {
      this.addEventListener(MouseEvent.DOUBLE_CLICK, this.tree_itemDoubleClick);
      this.addEventListener(ListEvent.ITEM_CLICK, this.tree_itemClick);
      this.addEventListener(KeyboardEvent.KEY_DOWN, this.itemDelete);
      BindingUtils.bindProperty(this, "selectedItem", DesignerModelLocator.getInstance().fileNavigatorModel, ["selectedItem"]);
      BindingUtils.bindProperty(this, "isEnabledDoubleClick", DesignerModelLocator.getInstance().fileNavigatorModel,
              ["isEnabledDoubleClick"])
    }


    /**
     * Handler of doubleClick event in a item of the tree control
     * If the current Item selected is a project expand the node
     * Else if the current Item selected is a process then open it into editor
     */
    public function tree_itemDoubleClick(event:MouseEvent):void {

      var node:Object = this.selectedItem;
      if (node != null && node is FileNavigatorProcess) {
        this.fileOpen();
      }
    }

    /**
     * Handler of click event in a item of the tree control
     * Sets the model selected item
     */
    public function tree_itemClick(event:ListEvent):void {
      DesignerModelLocator.getInstance().fileNavigatorModel.selectedItem = this.selectedItem as FileNavigatorItem;
    }

    /**
     * Remove a item in the parent item with the suppr keyboard
     */
    public function itemDelete(event:KeyboardEvent):void {
      if (event.keyCode == Keyboard.DELETE && this.selectedItem != null) {

        // remove temporaly keydown listener
        this.removeEventListener(KeyboardEvent.KEY_DOWN, this.itemDelete);

        var type:String;
        var fileItem:FileNavigatorItem = this.selectedItem as FileNavigatorItem
        if (fileItem is FileNavigatorProject) {
          type = "project";
        }
        else {
          type = "process";
        }

        deleteConfirmWindow = DeleteConfirmWindow(PopUpManager.createPopUp(Application.application as Application, DeleteConfirmWindow, true));
        deleteConfirmWindow.title = "Warning : " + type + " deletion..."

        if (fileItem is FileNavigatorProject) {
          deleteConfirmWindow.file.text = "Do you want to delete " + type + " '" + fileItem.name + "' (and all included " +
                  "processes)?\rThis operation can't be undone.";
        } else {
          deleteConfirmWindow.file.text = "Do you want to delete " + type + " '" + fileItem.name + "'?\rThis operation can't " +
                  "be undone.";
        }
        PopUpManager.centerPopUp(deleteConfirmWindow);
        deleteConfirmWindow.setFocus();
        deleteConfirmWindow.addEventListener(CloseEvent.CLOSE, itemDeleteHandler);
        deleteConfirmWindow.addEventListener(DeleteConfirmWindow.CANCEL, itemDeleteHandler);
      }
    }

    private function itemDeleteHandler(event:CloseEvent):void {
      if (event.type == CloseEvent.CLOSE) {
        if (this.selectedItem is FileNavigatorItem) {
          new FileNavigatorEvent(FileNavigatorEvent.ITEM_DELETE, selectedItem as FileNavigatorItem).dispatch();
        }
      }
      //add keydown listener temporaly deleted during deleteConfirmWindow modal windows opening
      this.addEventListener(KeyboardEvent.KEY_DOWN, this.itemDelete);
    }

    /**
     * Open the current selected item in the FileNavigator view if possible (ie a process)
     *
     */
    public function fileOpen():void {
      var process:FileNavigatorProcess = this.selectedItem as FileNavigatorProcess;

      var evt:FileNavigatorEvent = new FileNavigatorEvent(FileNavigatorEvent.PROCESS_OPEN, process);
      CairngormEventDispatcher.getInstance().dispatchEvent(evt);
    }

    /**
     * Initialize the list of the files show into fileNavigatorView.
     * This is done during the CreationComplete phase, because the ServiceLocator elements are not initialized before.
     *
     */
    public function initFiles(event:Event):void {
      new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL).dispatch();
    }

    override public function set selectedItem(value:Object):void {
      if (value is FileNavigatorProcess) {
        this.expandItem((value as FileNavigatorProcess).project, true);
      }
      else if (value is FileNavigatorProject) {
        this.expandItem((value as FileNavigatorProject), true);
      }
      super.selectedItem = value;
    }

    /**
     * Disabled double click to prevent open twice the same process.
     *
     */
    public function set isEnabledDoubleClick(value:Boolean):void {
      _isEnabledDoubleClick = value;
      if (_isEnabledDoubleClick)
         this.addEventListener(MouseEvent.DOUBLE_CLICK, this.tree_itemDoubleClick);
      else
         this.removeEventListener(MouseEvent.DOUBLE_CLICK, this.tree_itemDoubleClick);
    }
  }
}
